from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.models import User
from django.utils.translation import gettext_lazy as _

from accounts.models import Profile


class RegistrationForm(forms.ModelForm):
    password = forms.CharField(label=_("Password"), required=True, strip=False, widget=forms.PasswordInput)

    password_confirm = forms.CharField(label=_("ConfirmPassword"), required=True, strip=False,
                                       widget=forms.PasswordInput)

    email = forms.EmailField(label=_("Email"), required=True, widget=forms.EmailInput)

    def clean(self):
        clean_data = super().clean()
        password = clean_data.get("password")
        password_confirm = clean_data.get("password_confirm")
        first_name = clean_data.get("first_name")
        last_name = clean_data.get("last_name")
        if not first_name and not last_name:
            raise forms.ValidationError("Enter first name or name")
        if password and password_confirm and password != password_confirm:
            raise forms.ValidationError("Password are not equal")
        return clean_data

    def save(self, commit=True):
        user = super().save(commit=False)
        user.set_password(self.cleaned_data["password"])
        if commit:
            user.save()
        return user

    class Meta:
        model = User
        fields = ["username", "first_name", "last_name", "email", "password", "password_confirm"]
        labels = {"first_name": _("Name"), "last_name": _("Surname"), "email": _("Email"), "username": _("username"),
                  "password": _("Password"), "password_confirm": _("ConfirmPassword")}


class UserChangeForm(forms.ModelForm):
    class Meta:
        model = get_user_model()
        fields = ["first_name", "last_name", "email"]
        labels = {"first_name": _("Name"), "last_name": _("Surname"), "email": _("Email")}


class ProfileChangeForm(forms.ModelForm):
    birth_date = forms.DateField(widget=forms.TextInput(attrs={'placeholder': 'YYYY-MM-DD'}))

    class Meta:
        model = Profile
        exclude = ["user"]
        labels = {"birth_date": _("BirthDate")}


class PasswordChangeForm(forms.ModelForm):
    old_password = forms.CharField(label=_("OldPassword"), strip=False,
                                   widget=forms.PasswordInput)
    password = forms.CharField(label=_("Password"), strip=False,
                               widget=forms.PasswordInput)
    password_confirm = forms.CharField(label=_("ConfirmPassword"), strip=False,
                                       widget=forms.PasswordInput)

    def clean_password_confirm(self):
        password = self.cleaned_data["password"]
        password_confirm = self.cleaned_data["password_confirm"]
        if password and password_confirm and password != password_confirm:
            raise forms.ValidationError("Passwords are not equal")
        return password_confirm

    def clean_old_password(self):
        old_password = self.cleaned_data["old_password"]
        if not self.instance.check_password(old_password):
            raise forms.ValidationError("Wrong old password")
        return old_password

    def save(self, commit=True):
        user = self.instance
        user.set_password(self.cleaned_data["password"])
        if commit:
            user.save()

        return user

    class Meta:
        model = get_user_model()
        fields = ["old_password", "password", "password_confirm"]
        labels = {"old_password": _("OldPassword"), "password": _("Password"), "password_confirm": _("ConfirmPassword")}
